
/* Extracted from common, mem, and vector libraries originally written
   by Kanzelberger, Ristad, and Yianilos. */

typedef int 	OBJECT_TYPE;
typedef double	ALIGN;

extern void	error_fatal( char *function, char *message, ... );

typedef union memory_tag 
{
  struct { int bytes; } self;
  ALIGN                 align;
} memory_t;

extern int Memory_bytes_used;

#define MAX(x,y)    (((x) > (y)) ? (x) : (y))

#define  mem_total_space()   	(Memory_bytes_used)
         /* Returns: total number of allocated bytes, including headers */

void    *mem_alloc(int bytes);
         /* Returns: uninitialized block of storage of specified size */

void    *mem_realloc(void *mem, int bytes);
         /* Effect: destroys original block of memory if new block is allocated   */
         /* Returns: block of storage that is exact copy of mem of specified size */

void	 mem_free(void *mem);
         /* Effect: return specified block of memory to system */

void    *mem_move(void *destination, void *source, unsigned long length);
         /* Effect: moves specified number of bytes from source to destination */
         /* Returns: destination pointer                                       */

typedef void	vector_t;

typedef union vector_header_tag 
{
  struct {
    int	 object_size;
    int  object_type;
    int  length;          /* number of current entries */
    int  allocn;          /* max number of entries */
  }      self;
  ALIGN  align;
} vector_header_t;

#define vector_object_size(v)      (((vector_header_t *)(v) - 1)->self.object_size)
        /* Returns: size of individual vector entries, in bytes   */
#define vector_object_type(v)      (((vector_header_t *)(v) - 1)->self.object_type)
        /* Returns: initial user-defined object_type argument     */
#define vector_length(v)           (((vector_header_t *)(v) - 1)->self.length)
        /* Returns: number of components currently in vector      */

vector_t   *vector_create(int object_size, int object_type, int length);
            /* Arguments: object_type is user-defined type                 */
            /* Returns: vector_t with length objects of size object_size   */
void        vector_destroy(vector_t *vector);
            /* Effect: frees all memory used by given vectory              */

vector_t   *vector_extend(vector_t *vector, void *object_p);
            /* Effect: appends *object_p to end of vector                  */
            /* Returns: address of modified vector                         */
vector_t   *vector_contract(vector_t *vector, int n);
            /* Effect: deletes last n entries of vector                      */
            /* Returns: address of modified vector                           */





